DEFINITION MODULE SEProto;

(* SE-Protokoll zur Kommunikation zwischen Shells zur Programmentwicklung
 * und Editoren.
 *
 * Dieses Protokoll dient dazu, unter Multitasking-Systemen die Shell
 * zur Compilierung und hnlichen Sachen zu veranlassen, und um Fehler-
 * meldungen und/oder Fehlerdateien an den Editor zurckzugeben.
 *
 * Kommandos bzw. Nachrichten von der Shell zum Editor beginnen
 * mit SE_, Nachrichten vom Editor zur Shell beginnen mit ES_
 *
 * Da teilweise Zeiger auf Speicherbereiche bergeben werden, ist vom
 * jeweiligen Absender der Nachricht darauf zu achten, da diese
 * Speicherbereiche vom Empfnger auch lesbar sind (Memory Protection!).
 * Dafr sind diese als Readable zu allozieren!
 *
 *)

(*
 * Erweiterungen
 *
 * Datum    Version nderungen
 *
 * 30.01.96  1.01   SE_CLOSE fr Dateien/Masken, mit Close-Flag
 *                    msg[5] = 0 -> nur sichern
 *                    msg[5] = 1 -> sichern + schlieen
 *                  ES/SE_PROJEKT mit leerem Argument (NULL)
 *                  Um zur Version 0x100 kompatibel zu sein, sollte
 *                  das Programm die Versionsnummer berprfen!!
 *
 * 13.08.96  1.02   SE_OPEN mit Zeilen- und Spaltennummer
 *                  neue Nachricht SE_MENU
 *
 * 19.11.96  1.03   bei ES_COMPILE kann der Dateiname auch NULL sein
 *
 * 08.01.97  1.04   SE_CLOSE: Flag 2 (schlieen ohne sichern)
 *                  SE_MENU: progName
 *
 * 23.12.97  1.05   ES_SHLCTRL neu, zur Kontrolle der Shell
 *                  Dafr neuer String in SEMENUINFO.
 *)

EXPORT SE_INIT, SE_OK, SE_ACK, SE_OPEN, SE_ERROR, SE_ERRFILE, SE_PROJECT,
       SE_QUIT, SE_TERMINATE, SE_CLOSE,
       ES_INIT, ES_OK, ES_ACK, ES_COMPILE, ES_MAKE, ES_MAKEALL, ES_LINK,
       ES_EXEC, ES_MAKEEXEC, ES_PROJECT, ES_QUIT,
       SEPROTOVERSION,
       tShellCommands, tEditCommands, tShellSet, tEditSet,
       tPtrString, LONGBITSET,
       tErrInfo, tPtrErrInfo, tMessRec, tPtrMessRec;


CONST

    SE_INIT     =   4200H;
    (* Die Shell fragt an, ob ein Editor das SE-Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Als Antwort erhlt es ein ES_OK vom  Editor.
     *)
    SE_OK       =   4201H;
    (* Die Shell sagt dem Editor, das sie das Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     *)
    SE_ACK      =   4202H;
    (* Die Shell besttigt den Empfang eines Editorkommandos und gibt zurck,
     * ob das Kommando ausgefhrt wird.
     * Wort 3: TRUE: Kommando wird verstanden und ausgefhrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine SE_ACK mit TRUE sagt nichts darber aus, ob das Kommando erfolg-
     * reich ausgefhrt wurde. Es sagt nur, da die Shell das Kommando
     * versteht und ausfhren wird!
     *)
    SE_OPEN     =   4203H;
    (* Die Shell sagt dem Editor, da er einen Text ffnen
     * soll. Als Antwort erhlt die Shell ein ES_ACK
     * Wort 3+4: Ein Zeiger auf den Filenamen des zu ffnenden Files
     * Wort 5+6: Cursorposition (Zeilennummer)
     * Wort 7  : Cursorposition (Spaltennummer)
     *)
    SE_ERROR    =   4204H;
    (* Es ist ein Fehler beim Compilieren aufgetreten.
     * Wort 3+4: Ein Zeiger auf eine Infostruktur, die wie folgt
     *           aufgebaut ist:
     *              Wort 0+1: Ein Zeiger auf den Namen des compilierten Textes.
     *              Wort 2+3: Ein Zeiger auf den Text der Fehlermeldung
     *              Wort 4  : Die Fehlernummer
     *              Wort 5+6: Die fehlerhafte Zeile
     *              Wort 7  : Die Spalte mit dem Fehler (oder 0)
     * Mit ES_ACK besttigt der Editor die Meldung.
     *)
    SE_ERRFILE  =   4205H;
    (* Es sind Fehler aufgetreten. Die Fehlermeldungen stehen in einem
     * Errorfile, welches in der Message spezifiziert wird.
     * Wort 3+4: Ein Zeiger auf den Filenamen des Errorfiles mit den
     *           Fehlermeldungen
     * Wort 5+6: Ein Zeiger auf den Namen des compilierten Textes
     * Mit ES_ACK besttigt der Editor die Meldung.
     *)
    SE_PROJECT  =   4206H;
    (* Die Shell teilt dem Editor mit, da das Projekt gendert wurde.
     * Der Filename des aktuellen Projektfiles wird als Parameter bergeben
     * Wird NULL bergeben, sollte das aktuelle Projekt abgemeldet werden.
     * Wort 3+4: Ein Zeiger auf den Namen des Projektfiles oder NULL
     * Der Editor besttigt mit ES_ACK.
     * Eine vernnftige Reaktion des Editors wre es in dem Fall,
     * ebenfalls das Projekt zu wechseln, sofern er diese untersttzt.
     *)
    SE_QUIT     =   4207H;
    (* Die Shell teilt dem Editor mit, da sie jetzt beendet wird.
     * Der Editor sollte in dem Fall die Shell als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     *)
    SE_TERMINATE    = 4208H;
    (* Die Shell teilt dem Editor mit, da dieser sich beenden soll.
     * Der Editor sollte in dem Fall sich selbst beenden und seinen ganz
     * normalen Terminierungsproze durchlaufen (und in dem auch ein ES_QUIT
     * schicken!). Der Grund fr so eine Meldung der Shell knnte zum Beispiel
     * zu wenig Speicher zum compilieren sein.
     * Mit ES_ACK besttigt der Editor die Meldung.
     *)
    SE_CLOSE        = 4209H;
    (* Die Shell teilt dem Editor mit, da dieser bestimmte Texte sichern
     * bzw. schlieen soll. Beim Schlieen von genderten Texten sollte der
     * Editor vorher nachfragen.
     * Wort 3+4: Ein Zeiger auf den Namen einer Datei oder eine Dateimaske.
     *           '*.*' steht fr alle Textfenster (entspricht also dem SE_CLOSE
     *           der Protokoll-Version 0x100).
     * Wort 5  : 0 = nur sichern
     *           1 = sichern und schlieen
     * Mit ES_ACK besttigt der Editor die Meldung.
     *)
    SE_MENU         = 420AH;
    (* Die Shell teilte dem Editor mit, was er fr die einzelnen Programme
     * in sein Men eintragen soll.
     *
     * Word 3+4: Ein Zeiger auf eine MenuInfo-Struktur mit den Eintrgen.
     * Mit ES_ACK besttigt der Editor die Meldung.
     *)

    ES_INIT     =   4240H;
    (* Ein Editor fragt an, ob eine Shell das SE-Protokoll versteht.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Als Antwort erhlt es SE_OK von der Shell
     *)
    ES_OK       =   4241H;
    (* Der Editor beantwortet die Anfrage der Shell nach dem Protokoll.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     *)
    ES_ACK      =   4242H;
    (* Der Editor besttigt den Empfang des Kommandos
     * Wort 3: TRUE: Kommando wird verstanden und ausgefhrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine ES_ACK mit TRUE sagt nichts darber aus, ob das Kommando erfolg-
     * reich ausgefhrt wurde. Es sagt nur, da der Editor das Kommando
     * versteht und ausfhren wird!
     *)
    ES_COMPILE  =   4243H;
    (* Der Editor sagt der Shell, da sie ein File bersetzen soll.
     * Ein Pointer auf den Dateinamen wird in der Message bergeben.
     * Wort 3+4: Zeiger auf den Namen der zu compilierenden Datei
     *           (oder NULL).
     * Diese Nachricht mu mit SE_ACK besttigt werden.
     *)
    ES_MAKE     =   4244H;
    (* Der Editor sagt der Shell, da sie ein Make ausfhren soll.
     * Ein Filename kann in der Message bergeben werden, mu aber
     * nicht gesetzt sein und mu von der Shell auch nicht beachtet
     * werden!
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell besttigt mit SE_ACK.
     *)
    ES_MAKEALL  =   4245H;
    (* Der Editor sagt der Shell, da ein komplettes Make All ausgefhrt
     * werden soll. Ein Filename fr das Makefile kann (mu nicht) in
     * der Message bergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell besttigt mit SE_ACK.
     *)
    ES_LINK     =   4246H;
    (* Der Editor sagt der Shell, da das Programm
     * gelinkt werden soll. Ein Filename kann
     * in der Message bergeben werden, mu aber
     * nicht unbedingt beachtet werden von der Shell!
     * Wort 3+4: Zeiger auf den Namen der Source, die gelinkt werden soll
     *           (oder NULL)
     * Die Shell besttigt mit SE_ACK.
     *)
    ES_EXEC     =   4247H;
    (* Der Editor sagt der Shell, da das Programm
     * zu der Source ausgefhrt werden soll. Ein
     * Filename kann bergeben werden, mu von der
     * Shell aber nicht beachtet werden.
     * Wort 3+4: Zeiger auf den Namen des auszufhrenden Files (oder NULL)
     *           Bei einer Sourcedatei ist diese ggf. noch zu
     *           compilieren und/oder zu linken
     * Die Shell besttigt mit SE_ACK.
     *)
    ES_MAKEEXEC =   4248H;
    (* Die Shell soll ein Make ausfhren und danach das Programm ausfhren.
     * Ein Filename fr das Makefile kann (mu nicht) in der Message
     * bergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell besttigt mit SE_ACK.
     *)
    ES_PROJECT  =   4249H;
    (* Der Editor teilt der Shell mit, da das Project gendert/gewechselt
     * wurde. Der Filename des  Projektfiles wird als Parameter in der Message
     * bergeben. Wird NULL bergeben, sollte das aktuelle Projekt abgemeldet
     * werden.
     * Wort 3+4: Zeiger auf den Namen des Projektfiles oder NULL.
     * Die Shell besttigt mit SE_ACK.
     * Eine vernnftige Reaktion der Shell wre in dem Fall, ebenfalls
     * das Projekt zu wechseln, sofern sie diese untersttzt.
     *)
    ES_QUIT     =   424AH;
    (* Der Editor teilt der Shell mit, da er jetzt beendet wird.
     * Die Shell sollte in dem Fall den Editor als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     *)

    ES_SHLCTRL  =   424BH;
    (*
     * Sonstige Kontrolle der Shell durch den Editors.
     * Wort 3+4    Zeiger auf den Namen des obersten Fensters oder NULL.
     * Wort 5      Steuer-Flag:
     *                0: Shell soll sich toppen (Men/Fenster).
     * Die Shell besttigt mit SE_ACK.
     *)

    SE_DATA     =   0;
    (* Nur fr Typdeklaration benutzt, eigentlich berflssig
     *)

    SEPROTOVERSION  = 105H; (* Die Versionsnummer des Protokolls.
                             * Highbyte ist die Major Number
                             * Lowbyte ist die Lower Number
                             *)

(* Typdefinitionen fr die Nachrichten
 *)

(* Dies definiert die einzelnen Bits, welche Nachrichten untersttzt werden
 *)
TYPE    tShellCommands = (seInit, seOk, seAck, seOpen, seError, seErrfile,
                          seProject, seQuit, seTerminate, seClose, seMenu );

        tEditCommands  = (esInit, esOk, esAck, esCompile, esMake, esMakeall,
                          esLink, esExec, esMakeexec, esProject, esQuit, esShlctrl,
                          res12, res13, res14, res15, res16, res17, res18,
                          res19, res20, res21, res22, res23, res24, res25,
                          res26, res27, res28, res29, res30, res31);

        tShellSet      = SET OF tShellCommands;

        tEditSet       = SET OF tEditCommands;

        tCtrlCommands  = ( topShell );

        tCtrlSet       = SET OF tCtrlCommands;

        tString        = ARRAY [0..255] OF CHAR;

        tPtrString  = POINTER TO ARRAY [0..255] OF CHAR;
        (* Zeiger auf einen nullterminierten String
         *)

        LONGBITSET  = SET OF [0..31];
        (* Leider kennt das nicht jeder Compiler
         *)

        tErrInfo     = RECORD
                        errFile : tPtrString;
                        (* Zeiger auf den Namen der compilierten Datei
                         *)
                        errMess : tPtrString;
                        (* Zeiger auf die Fehlermeldung
                         *)
                        errNum  : INTEGER;
                        (* Die Fehlernummer
                         *)
                        errLine : LONGINT;
                        (* Die fehlerhafte Zeile
                         *)
                        errRow  : INTEGER;
                        (* Die Spalte mit dem Fehler (oder 0)
                         *)
                      END;
        tPtrErrInfo = POINTER TO tErrInfo;

        tMenuInfo    = RECORD
                        compStr : tPtrString;
                        makeStr : tPtrString;
                        makeAllStr : tPtrString;
                        linkStr : tPtrString;
                        execStr : tPtrString;
                        makeExecStr : tPtrString;
                        progName : tPtrString;
                        shellCtrlString : tPtrString;
                       END;
        tPtrMenuInfo = POINTER TO tMenuInfo;

        (* ber diese Recorddefinition kann man alle
         * Messages des Protokolls behandeln
         *)

        tMessRec    = RECORD
                        msg     : INTEGER;
                        (* Messagenummer, Wort 0 *)
                        apId    : INTEGER;
                        (* Applikation ID des Senders, Wort 1 *)
                        overLen : INTEGER;
                        (* berlnge der Nachricht (immer 0 fr SE-Protokoll),
                         * Wort 2 *)
                        CASE : CARDINAL OF
                          SE_INIT,
                          ES_INIT   : shellBits: tShellSet;
                          (* Bits, die angeben, welche Shell-Nachrichten
                           * untersttzt werden (Wort 3)
                           *)
                                      editBits : tEditSet;
                          (* Bits, die angeben, welche Editor-Nachrichten
                           * untersttzt werden (Worte 4 und 5)
                           *)
                                      version  : CARDINAL;
                          (* Untersttze Versionsnummer des Protokolls
                           * Aktuell: $102
                           *)
                        | SE_OK,
                          ES_OK     : okShellBits: tShellSet;
                          (* Bits, die angeben, welche Shell-Nachrichten
                           * untersttzt werden (Wort 3)
                           *)
                                      okEditBits : tEditSet;
                          (* Bits, die angeben, welche Editor-Nachrichten
                           * untersttzt werden (Worte 4 und 5)
                           *)
                                      okVersion: CARDINAL;
                          (* Untersttze Versionsnummer des Antwortenden
                           * Aktuell: $102
                           *)
                                      otherId  : INTEGER;
                          (* Die ApId des Senders, auf das dies eine Antwort
                           * ist (Wort 7)
                           *)
                        | SE_ACK,
                          ES_ACK    : acknowledge:  BOOLEAN;
                          (* TRUE: Kommando wurde akzeptiert,
                           * FALSE: Kommando konnte nicht ausgefhrt werden
                           *)
                        | SE_OPEN   : namePtr   : tPtrString;
                          (* Zeiger auf den Namen des zu ladenden Textes *)
                                      errLine : LONGINT;
                          (* Die fehlerhafte Zeile *)
                                      errRow  : INTEGER;
                          (* Die Spalte mit dem Fehler (oder 0) *)
                        | SE_ERROR  : errorPtr  : tPtrErrInfo;
                          (* Zeiger auf Speicherbereich, der die Fehlermeldung
                           * spezifiziert.
                           *)
                        | SE_ERRFILE : errPtr   : tPtrString;
                          (* Zeiger auf den Namen des Errorfiles *)
                                       fnamePtr : tPtrString;
                          (* Zeiger auf den Namen der compilierten Datei *)
                        | SE_CLOSE   : maskPtr  : tPtrString;
                          (* Zeiger auf den Namen einer Datei oder eine
                           * Dateimaske ('*.*' steht fr alle Textfenster)
                           *)
                                       closeFlag : BOOLEAN;
                        | SE_MENU    : menuPtr  : tPtrMenuInfo;
                          (* Zeiger auf Speicherbereich, der die Mentexte
                           * spezifiziert.
                           *)
                        | SE_PROJECT,
                          ES_COMPILE,
                          ES_MAKE,
                          ES_MAKEALL,
                          ES_LINK,
                          ES_EXEC,
                          ES_MAKEEXEC,
                          ES_PROJECT : filePtr  : tPtrString;
                          (* Zeiger auf den Filenamen des Files, welches
                           * behandelt werden soll
                           *)
                        | SE_QUIT,
                          SE_TERMINATE,
                          ES_QUIT,
                          SE_DATA:     data     : ARRAY [0..4] OF INTEGER;
                          (* Definition fr Maximallnge des Records
                           *)
                        | ES_SHLCTRL:  ctrlBits : tCtrlSet;
                        END;
                      END;

        tPtrMessRec = POINTER TO tMessRec;
        (* Ein Zeiger auf diese Messageblock
         *)

END SEProto.
