MODULE SEProtocol;

IMPORT Str:=Strings;

(* SE-Protokoll zur Kommunikation zwischen Shells zur Programmentwicklung
 * und Editoren.
 *
 * Dieses Protokoll dient dazu, unter Multitasking-Systemen die Shell
 * zur Compilierung und hnlichen Sachen zu veranlassen, und um Fehler-
 * meldungen und/oder Fehlerdateien an den Editor zurckzugeben.
 *
 * Kommandos bzw. Nachrichten von der Shell zum Editor beginnen
 * mit SE, Nachrichten vom Editor zur Shell beginnen mit ES
 *
 * Da teilweise Zeiger auf Speicherbereiche bergeben werden, ist vom
 * jeweiligen Absender der Nachricht darauf zu achten, da diese
 * Speicherbereiche vom Empfnger auch lesbar sind (Memory Protection!).
 * Dafr sind diese als Readable zu allozieren!
 *
 *
 * Erweiterungen
 *
 * Datum      Version     nderungen
 *
 * 27.01.96   101         SE_CLOSE fr Dateien oder Masken
 *                          Wort 5 : 0 = nur sichern
 *                                   1 = sichern und schlieen
 *                        ES/SE_PROJEKT mit leerem Argument
 * 17.07.96   102         SE_MENU definiert
 * 30.07.96               SE_OPEN mit Cursor-Pos.
 *
 * 19.11.96   103         ES_COMPILER auch ohne Argument
 *
 * 08.01.97  1.04   SE_CLOSE: Flag 2 (schlieen ohne sichern)
 *                  SE_MENU: progName
 *
 * 23.12.97  1.05   ES_SHLCTRL neu, zur Kontrolle der Shell
 *                  Dafr neuer String in SEMENUINFO.
 *)


CONST

    SEPROTOVERSION *= 105H;

    SEINIT     *=   4200H;
    (* Die Shell fragt an, ob ein Editor das SE-Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Als Antwort erhlt es ein ESOK vom  Editor.
     *)
    SEOK       *=   4201H;
    (* Die Shell sagt dem Editor, das sie das Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     *)
    SEACK      *=   4202H;
    (* Die Shell besttigt den Empfang eines Editorkommandos und gibt zurck,
     * ob das Kommando ausgefhrt wird.
     * Wort 3: TRUE: Kommando wird verstanden und ausgefhrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine SEACK mit TRUE sagt nichts darber aus, ob das Kommando erfolg-
     * reich ausgefhrt wurde. Es sagt nur, da die Shell das Kommando
     * versteht und ausfhren wird!
     *)
    SEOPEN     *=   4203H;
    (* Die Shell sagt dem Editor, da er einen Text ffnen
     * soll. Als Antwort erhlt die Shell ein ESACK
     * Wort 3+4: Ein Zeiger auf den Filenamen des zu ffnenden Files
     * Wort 5+6: Zeilennummer oder null
     * Wort 7  : Spaltennummer oder null
     *)
    SEERROR    *=   4204H;
    (* Es ist ein Fehler beim Compilieren aufgetreten.
     * Wort 3+4: Ein Zeiger auf eine Infostruktur, die wie folgt
     *           aufgebaut ist:
     *              Wort 0+1: Ein Zeiger auf den Namen des compilierten Textes.
     *              Wort 2+3: Ein Zeiger auf den Text der Fehlermeldung
     *              Wort 4  : Die Fehlernummer
     *              Wort 5+6: Die fehlerhafte Zeile
     *              Wort 7  : Die Spalte mit dem Fehler (oder 0)
     * Mit ESACK besttigt der Editor die Meldung.
     *)
    SEERRFILE  *=   4205H;
    (* Es sind Fehler aufgetreten. Die Fehlermeldungen stehen in einem
     * Errorfile, welches in der Message spezifiziert wird.
     * Wort 3+4: Ein Zeiger auf den Filenamen des Errorfiles mit den
     *           Fehlermeldungen
     * Wort 5+6: Ein Zeiger auf den Namen des compilierten Textes
     * Mit ESACK besttigt der Editor die Meldung.
     *)
    SEPROJECT  *=   4206H;
    (* Die Shell teilt dem Editor mit, da das Projekt
     * gendert wurde. Der Filename des aktuellen
     * Projektfiles wird als Parameter bergeben
     * Wort 3+4: Ein Zeiger auf den Namen des Projektfiles (oder NULL!)
     * Der Editor besttigt mit ESACK.
     * Eine vernnftige Reaktion des Editors wre es in dem Fall,
     * ebenfalls das Projekt zu wechseln, sofern er diese untersttzt.
     *)
    SEQUIT     *=   4207H;
    (* Die Shell teilt dem Editor mit, da sie jetzt beendet wird.
     * Der Editor sollte in dem Fall die Shell als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     *)
    SETERMINATE    *= 4208H;
    (* Die Shell teilt dem Editor mit, da dieser sich beenden soll.
     * Der Editor sollte in dem Fall sich selbst beenden und seinen ganz
     * normalen Terminierungsproze durchlaufen (und in dem auch ein ESQUIT
     * schicken!). Der Grund fr so eine Meldung der Shell knnte zum Beispiel
     * zu wenig Speicher zum compilieren sein.
     * Mit ESACK besttigt der Editor die Meldung.
     *)
    SECLOSE        *= 4209H;
    (* Die Shell teilt dem Editor mit, da dieser alle Texte schlieen soll.
     * Der Editor sollte bei genderten Texten vorher nachfragen.
     * Wort 3+4: Ein Zeiger auf den Namen einer Datei oder eine Dateimaske.
     *           Bei '*.*', alle Textfenster.
     * Wort 5:   0 = nur sichern, 1 = sicher und schlieen
     * Mit ESACK besttigt der Editor die Meldung.
     *)
    SEMENU         *= 420AH;
    (* Die Shell teilt dem Editor mit, was er in sein Men fr die einzelnen
     * Programme eintragen soll.
     * Wort 3+4: Ein Zeiger auf die entsprechende Struktur (s.u.)
     *)

    ESINIT     *=   4240H;
    (* Ein Editor fragt an, ob eine Shell das SE-Protokoll versteht.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Als Antwort erhlt es SEOK von der Shell
     *)
    ESOK       *=   4241H;
    (* Der Editor beantwortet die Anfrage der Shell nach dem Protokoll.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Untersttze Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     *)
    ESACK      *=   4242H;
    (* Der Editor besttigt den Empfang des Kommandos
     * Wort 3: TRUE: Kommando wird verstanden und ausgefhrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine ESACK mit TRUE sagt nichts darber aus, ob das Kommando erfolg-
     * reich ausgefhrt wurde. Es sagt nur, da der Editor das Kommando
     * versteht und ausfhren wird!
     *)
    ESCOMPILE  *=   4243H;
    (* Der Editor sagt der Shell, da sie ein File bersetzen soll.
     * Ein Pointer auf den Dateinamen wird in der Message bergeben.
     * Wort 3+4: Zeiger auf den Namen der zu compilierenden Datei (oder NIL).
     * Diese Nachricht mu mit SEACK besttigt werden.
     *)
    ESMAKE     *=   4244H;
    (* Der Editor sagt der Shell, da sie ein Make ausfhren soll.
     * Ein Filename kann in der Message bergeben werden, mu aber
     * nicht gesetzt sein und mu von der Shell auch nicht beachtet
     * werden!
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NIL)
     * Die Shell besttigt mit SEACK.
     *)
    ESMAKEALL  *=   4245H;
    (* Der Editor sagt der Shell, da ein komplettes Make All ausgefhrt
     * werden soll. Ein Filename fr das Makefile kann (mu nicht) in
     * der Message bergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NIL)
     * Die Shell besttigt mit SEACK.
     *)
    ESLINK     *=   4246H;
    (* Der Editor sagt der Shell, da das Programm
     * gelinkt werden soll. Ein Filename kann
     * in der Message bergeben werden, mu aber
     * nicht unbedingt beachtet werden von der Shell!
     * Wort 3+4: Zeiger auf den Namen der Source, die gelinkt werden soll
     *           (oder NULL)
     * Die Shell besttigt mit SEACK.
     *)
    ESEXEC     *=   4247H;
    (* Der Editor sagt der Shell, da das Programm
     * zu der Source ausgefhrt werden soll. Ein
     * Filename kann bergeben werden, mu von der
     * Shell aber nicht beachtet werden.
     * Wort 3+4: Zeiger auf den Namen des auszufhrenden Files (oder NIL)
     *           Bei einer Sourcedatei ist diese ggf. noch zu
     *           compilieren und/oder zu linken
     * Die Shell besttigt mit SEACK.
     *)
    ESMAKEEXEC *=   4248H;
    (* Die Shell soll ein Make ausfhren und danach das Programm ausfhren.
     * Ein Filename fr das Makefile kann (mu nicht) in der Message
     * bergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NIL)
     * Die Shell besttigt mit SEACK.
     *)
    ESPROJECT  *=   4249H;
    (* Der Editor teilt der Shell mit, da das Project
     * gendert/gewechselt wurde. Der Filename des
     * Projektfiles wird als Parameter in der Message
     * bergeben.
     * Wort 3+4: Zeiger auf den Namen des Projektfiles (oder NIL!)
     * Die Shell besttigt mit SEACK.
     * Eine vernnftige Reaktion der Shell wre in dem Fall, ebenfalls
     * das Projekt zu wechseln, sofern es diese untersttzt.
     *)
    ESQUIT     *=   424AH;
    (* Der Editor teilt der Shell mit, da er jetzt beendet wird.
     * Die Shell sollte in dem Fall den Editor als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     *)
    ES_SHLCTRL *=   424BH;
    (*
     * Sonstige Kontrolle der Shell durch den Editors.
     * Wort 3+4    Zeiger auf den Namen des obersten Fensters oder NULL.
     * Wort 5      Steuer-Flag:
     *                0: Shell soll sich toppen (Men/Fenster).
     * Die Shell besttigt mit SE_ACK.
     *)

    SEDATA     *=   0;
    (* Nur fr Typdeklaration benutzt, eigentlich berflssig
     *)


(* Die Bits fr SE_INIT/ES_INIT *)
    seINIT     *= 0;
    seOK       *=1;
    seACK      *=2;
    seOPEN     *=3;
    seERROR    *=4;
    seERRFILE  *=5;
    sePROJECT  *=6;
    seQUIT     *=7;
    seTERMINATE*=8;
    seCLOSE    *=9;
    seMENU     *=10;

    esINIT     *=0;
    esOK       *=1;
    esACK      *=2;
    esCOMPILE  *=3;
    esMAKE     *=4;
    esMAKEALL  *=5;
    esLINK     *=6;
    esEXEC     *=7;
    esMAKEEXEC *=8;
    esPROJECT  *=9;
    esQUIT     *=10;
    esSHLCTRL  *=11;


TYPE
    ShellSet*   = SET;
    EditSet*    = SET;

    ErrInfo*    = RECORD
                    errFile* : Str.ptr;   (* Zeiger auf den Namen der compilierten Datei *)
                    errMess* : Str.ptr;   (* Zeiger auf die Fehlermeldung *)
                    errNum*  : INTEGER;   (* Die Fehlernummer *)
                    errLine* : LONGINT;   (* Die fehlerhafte Zeile *)
                    errRow*  : INTEGER;   (* Die Spalte mit dem Fehler (oder 0) *)
                  END;
    PtrErrInfo* = POINTER TO ErrInfo;

    MenuInfo*   = RECORD
                    compStr* : Str.ptr;
                    makeStr* : Str.ptr;
                    makeAllStr* : Str.ptr;
                    linkStr* : Str.ptr;
                    execStr* : Str.ptr;
                    makeExecStr* : Str.ptr;
                    progName* : Str.ptr;
                    shellCtrlStr* : Str.ptr;
                  END(*RECORD*);
    PtrMenuInfo*= POINTER TO MenuInfo;

END SEProtocol.
